import SwiftUI
import Logging
import LiveKit
import KeychainAccess

public let sync = ValueStore<Preferences>(store: Keychain(service: "com.globekeeper.liveKit"),
                                   key: "preferences",
                                   default: Preferences())

public struct RoomContextView: View {
  
  public init() {
  }
  

    @EnvironmentObject var appCtx: AppContext
    @StateObject var roomCtx = RoomContext(store: sync)

    var shouldShowRoomView: Bool {
        roomCtx.room.room.connectionState.isConnected || roomCtx.room.room.connectionState.isReconnecting
    }

    func computeTitle() -> String {
        if shouldShowRoomView {
            let elements = [roomCtx.room.room.name,
                            roomCtx.room.room.localParticipant?.name,
                            roomCtx.room.room.localParticipant?.identity]
            return elements.compactMap { $0 }.filter { !$0.isEmpty }.joined(separator: " ")
        }

        return "LiveKit"
    }

  public var body: some View {
        ZStack {
            Color.black
                .ignoresSafeArea()

            if shouldShowRoomView {
                RoomView()
            } else {
                ConnectView()
            }

        }
        .environment(\.colorScheme, .dark)
        .foregroundColor(Color.white)
        .environmentObject(roomCtx)
        .environmentObject(roomCtx.room)
        .navigationTitle(computeTitle())
        .onDisappear {
            print("\(String(describing: type(of: self))) onDisappear")
            Task {
                try await roomCtx.disconnect()
            }
        }
        .onOpenURL(perform: { url in

            guard let urlComponent = URLComponents(url: url, resolvingAgainstBaseURL: false) else { return }
            guard let host = url.host else { return }

            let secureValue = urlComponent.queryItems?.first(where: { $0.name == "secure" })?.value?.lowercased()
            let secure = ["true", "1"].contains { $0 == secureValue }

            let tokenValue = urlComponent.queryItems?.first(where: { $0.name == "token" })?.value ?? ""

            var builder = URLComponents()
            builder.scheme = secure ? "wss" : "ws"
            builder.host = host
            builder.port = url.port

            guard let builtUrl = builder.url?.absoluteString else { return }

            print("built URL: \(builtUrl), token: \(tokenValue)")

            Task { @MainActor in
                roomCtx.url = builtUrl
                roomCtx.token = tokenValue
                if !roomCtx.token.isEmpty {
                    let room = try await roomCtx.connect()
                    appCtx.connectionHistory.update(room: room)
                }
            }
        })
    }
}
